from ._security import (
    generate_keys,
    sign_packet,
    verify_packet,
    encrypt_packet,
    decrypt_packet,
)

__all__ = [
    "generate_keys",
    "sign_packet",
    "verify_packet",
    "encrypt_packet",
    "decrypt_packet",
]



from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend
from cryptography.exceptions import InvalidSignature


def generate_keys(print_keys: bool = False):
    """Generate RSA private and public keys."""
    private_key = rsa.generate_private_key(public_exponent=65537, key_size=2048)
    public_key = private_key.public_key()

    # If true, print them in the .env format
    if print_keys:
        private_key_str = (
            private_key.private_bytes(
                serialization.Encoding.PEM,
                serialization.PrivateFormat.PKCS8,
                serialization.NoEncryption(),
            )
            .decode()
            .replace("\n", "\\n")
        )

        public_key_str = (
            public_key.public_bytes(
                serialization.Encoding.PEM, serialization.PublicFormat.PKCS1
            )
            .decode()
            .replace("\n", "\\n")
        )

        print(f"PRIVATE_KEY={private_key_str}")
        print(f"PUBLIC_KEY={public_key_str}")
    return private_key, public_key


def load_private_key(private_key):
    """Load a private key from a PEM string or return the key if it's already an object."""
    if isinstance(private_key, str):
        private_key = serialization.load_pem_private_key(
            private_key.encode(),
            password=None,
            backend=default_backend()
        )
    return private_key


def load_public_key(public_key):
    """Load a public key from a PEM string or return the key if it's already an object."""
    if isinstance(public_key, str):
        public_key = serialization.load_pem_public_key(
            public_key.encode(),
            backend=default_backend()
        )
    return public_key


def sign_packet(packet: bytes, private_key):
    """Sign a packet using the provided private key."""
    private_key = load_private_key(private_key)
    packet_hash = hashes.Hash(hashes.SHA256())
    packet_hash.update(packet)
    digest = packet_hash.finalize()

    signature = private_key.sign(
        digest,
        padding.PSS(
            mgf=padding.MGF1(hashes.SHA256()), salt_length=padding.PSS.MAX_LENGTH
        ),
        hashes.SHA256(),
    )
    return signature


def verify_packet(packet: bytes, signature: bytes, public_key):
    """Verify a packet using the provided public key."""
    public_key = load_public_key(public_key)
    packet_hash = hashes.Hash(hashes.SHA256())
    packet_hash.update(packet)
    digest = packet_hash.finalize()

    try:
        public_key.verify(
            signature,
            digest,
            padding.PSS(
                mgf=padding.MGF1(hashes.SHA256()), salt_length=padding.PSS.MAX_LENGTH
            ),
            hashes.SHA256(),
        )
        return True
    except InvalidSignature:
        return False


def encrypt_packet(packet: bytes, public_key):
    """Encrypt a packet using the recipient's public key."""
    public_key = load_public_key(public_key)
    encrypted_packet = public_key.encrypt(
        packet,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None,
        ),
    )
    return encrypted_packet


def decrypt_packet(encrypted_packet: bytes, private_key):
    """Decrypt a packet using the recipient's private key."""
    private_key = load_private_key(private_key)
    decrypted_packet = private_key.decrypt(
        encrypted_packet,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None,
        ),
    )
    return decrypted_packet


if __name__ == "__main__":
    # Example usage
    private_key, public_key = generate_keys(print_keys=True)
    packet = b"Important data from API"

    signature = sign_packet(packet, private_key)
    is_verified = verify_packet(packet, signature, public_key)
    print("Packet is verified as coming from the API:", is_verified)

    encrypted_packet = encrypt_packet(packet, public_key)
    decrypted_packet = decrypt_packet(encrypted_packet, private_key)
    print("Decrypted packet:", decrypted_packet)



"""
Current Limitations:
1. Streaming only works for one command at a time.
2. It only streams stdout, not stderr.
3. It requires sending the session_id with every request. (This will change.)

Todo:
1. Make it stream stderr too...
"""

import socketio
import asyncio
import aiohttp
from pydantic import BaseModel
from typing import Literal, Optional, AsyncIterator, Union
from enum import Enum


class CommandMode(str, Enum):
    STREAM = "stream"
    WAIT = "wait"
    BACKGROUND = "background"


class CommandOutput(BaseModel):
    output: str
    type: Literal["stdout", "stderr"]
    process_id: Union[str, int]


class CommandExit(BaseModel):
    exit_code: int
    process_id: Union[str, int]


class CommandResult(BaseModel):
    stdout: str | None = None
    stderr: str | None = None
    exit_code: int | None = None
    finished: bool


class Status(BaseModel):
    running: bool | None


class CommandKilled(BaseModel):
    status: str
    exit_code: Optional[int] = None


class CommandError(BaseModel):
    error: str


class BackgroundProcess(BaseModel):
    process_id: Union[str, int]


class SandboxClient:
    def __init__(self, url: str = "http://localhost:80"):
        self.url = url
        self.sio = socketio.AsyncClient()
        self.sessions = {}
        self.initialized_event = asyncio.Event()
        self.stream_queue = asyncio.Queue()
        self.result_event = asyncio.Event()
        self.result_data = None

        # Register event handlers
        self.sio.on("connect", self.on_connect)
        self.sio.on("disconnect", self.on_disconnect)
        self.sio.on("initialized", self.on_initialized)
        self.sio.on("command_output", self.on_command_output)
        self.sio.on("command_exit", self.on_command_exit)
        self.sio.on("command_result", self.on_command_result)
        self.sio.on("status", self.on_status)
        self.sio.on("command_killed", self.on_killed)
        self.sio.on("command_error", self.on_error)

    async def on_connect(self):
        print("Connected to the server")

    async def on_disconnect(self):
        print("Disconnected from the server")

    async def on_initialized(self, data):
        print("Initialization response:", data)
        self.initialized_event.set()

    async def on_command_output(self, data):
        await self.stream_queue.put(CommandOutput(**data))

    async def on_command_exit(self, data):
        exit_info = CommandExit(**data)
        print(
            f"Command exited with code {exit_info.exit_code} (Process ID: {exit_info.process_id})"
        )
        await self.stream_queue.put(None)  # Signal the end of the stream

    async def on_command_result(self, data):
        self.result_data = CommandResult(**data)
        self.result_event.set()

    async def on_status(self, data):
        status = Status(**data)
        print("Status:", status)

    async def on_killed(self, data):
        killed_info = CommandKilled(**data)
        print("Command killed:", killed_info)

    async def on_error(self, data):
        error_info = CommandError(**data)
        print("Error:", error_info)

    async def connect(self):
        await self.sio.connect(self.url, transports=["websocket"])

    async def initialize_session(self, session_id: str, path: str):
        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.url}/initialize",
                json={"session_id": session_id, "path": path},
            ) as response:
                init_response = await response.json()
                self.sessions[session_id] = init_response["session_id"]

        await self.sio.emit("initialize", {"session_id": session_id})
        await self.initialized_event.wait()

    async def run_command(
        self,
        session_id: str,
        command: str,
        mode: CommandMode = CommandMode.STREAM,
        path: Optional[str] = None,
        timeout: Optional[int] = None,
    ) -> Union[AsyncIterator[CommandOutput], CommandResult, BackgroundProcess]:
        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.url}/run_command",
                json={
                    "session_id": session_id,
                    "command": command,
                    "mode": mode,
                    "path": path,
                    "timeout": timeout,
                },
            ) as response:
                result = await response.json()

        if mode == CommandMode.STREAM:
            process_id = result["process_id"]
            await self.sio.emit(
                "start_command_stream",
                {"session_id": session_id, "process_id": process_id},
            )
            return self._stream_output()
        elif mode == CommandMode.WAIT:
            return CommandResult(**result)
        elif mode == CommandMode.BACKGROUND:
            return BackgroundProcess(process_id=result["process_id"])

    async def _stream_output(self) -> AsyncIterator[CommandOutput]:
        while True:
            output = await self.stream_queue.get()
            if output is None:  # End of stream
                break
            yield output

    async def check_status(self, session_id: str, process_id: str) -> Status:
        # Create an event to wait for the status response
        status_event = asyncio.Event()
        status_data = {}

        async def on_status(data):
            nonlocal status_data
            status_data = data
            status_event.set()

        # Temporarily override the on_status handler
        self.sio.on("status", on_status)

        # Emit the check_status event with session_id and process_id
        await self.sio.emit(
            "check_status", {"session_id": session_id, "process_id": process_id}
        )

        # Wait for the status response
        await status_event.wait()

        # Restore the original on_status handler
        self.sio.on("status", self.on_status)

        return Status(**status_data)

    async def kill_command(self, session_id: str, process_id: str) -> CommandKilled:
        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.url}/kill_command",
                json={"session_id": session_id, "process_id": process_id},
            ) as response:
                result = await response.json()

        # Ensure the result includes a status field
        if "status" not in result:
            result["status"] = "not found"

        return CommandKilled(**result)

    async def disconnect(self):
        await self.sio.disconnect()
        # Ensure all sessions are closed
        await self.sio.eio.disconnect()

    async def get_file(self, session_id: str, file_path: str) -> str:
        async with aiohttp.ClientSession() as session:
            async with session.get(
                f"{self.url}/get_file",
                params={"session_id": session_id, "file_path": file_path},
            ) as response:
                if response.status == 200:
                    return await response.text()
                else:
                    raise Exception(f"Failed to get file: {response.status}")

    async def get_all_file_paths(self, session_id: str) -> list[str]:
        async with aiohttp.ClientSession() as session:
            async with session.get(
                f"{self.url}/get_all_file_paths",
                params={"session_id": session_id},
            ) as response:
                if response.status == 200:
                    return await response.json()
                else:
                    raise Exception(f"Failed to get file paths: {response.status}")


if __name__ == "__main__":

    async def main():
        client = SandboxClient(url="http://localhost:80")
        await client.connect()

        # Initialize a session
        await client.initialize_session("your-session-id-here", "test_path")

        result = await client.run_command(
            "your-session-id-here",
            "git clone https://github.com/lukejagg/ichi-site.git test_path",
            mode=CommandMode.WAIT,
        )
        print("Result:", result)

        result = await client.run_command(
            "your-session-id-here", "ls -la", mode=CommandMode.WAIT, path="test_path"
        )
        print(result.stdout)

        result = await client.run_command(
            "your-session-id-here",
            "npm install",
            mode=CommandMode.STREAM,
            path="test_path",
        )
        async for output in result:
            print(output.output)

        result = await client.run_command(
            "your-session-id-here",
            "npm run start",
            mode=CommandMode.STREAM,
            path="test_path",
            # timeout=200,
        )
        async for output in result:
            print(output.output)

        return

        # Run a command in STREAM mode
        async for output in await client.run_command(
            "your-session-id-here", "echo Hello, World!", mode=CommandMode.STREAM
        ):
            print("Streamed Output:", output)

        # Run a command in WAIT mode
        for i in range(10):
            result = await client.run_command(
                "your-session-id-here", "echo HELLO WORLD!", mode=CommandMode.WAIT
            )
            print("Command Result:", result)

        # Run a command in BACKGROUND mode
        background_process = await client.run_command(
            "your-session-id-here",
            "echo Hello, World! && sleep 10",
            mode=CommandMode.BACKGROUND,
        )
        print("Background Process ID:", background_process.process_id)

        # Check the status of the command
        status = await client.check_status(
            "your-session-id-here", background_process.process_id
        )
        print("Status:", status)

        # Kill the command
        killed_info = await client.kill_command(
            "your-session-id-here", background_process.process_id
        )
        print("Killed Info:", killed_info)

        # Example usage of get_file
        try:
            file_content = await client.get_file("your-session-id-here", "LICENSE")
            print("File Content:", file_content)
        except Exception as e:
            print(e)

        # Example usage of get_all_file_paths
        try:
            file_paths = await client.get_all_file_paths("your-session-id-here")
            print("File Paths:", file_paths)
        except Exception as e:
            print(e)

        # Disconnect from the server
        await client.disconnect()

    # Run the main function
    asyncio.run(main())



"""
Limitations:
1. Cannot whitelist IPs for this service (for only client to be able to access)
2. All ports open by default (instead, could map the port dynamically)? [may be fixed on the service, not the pod]
"""

import base64
import json
import logging
import asyncio
import time  # Ensure this import is at the top of the file
from typing import Optional, Tuple  # Added logging package
from kubernetes import client, config
from kubernetes.client.rest import ApiException


class KubeboxPodExistsError(Exception):
    """Exception raised when a pod already exists."""


class KubeboxPod:
    def __init__(self, name: str, namespace: str, kubebox: "Kubebox" = None):
        self.name = name
        self.namespace = namespace
        self._kubebox = kubebox

    async def wait_until_ready(self, poll_interval: float = 0.1):
        start_time = asyncio.get_event_loop().time()
        while True:
            try:
                pod = await asyncio.to_thread(
                    self._kubebox._core_v1.read_namespaced_pod,
                    name=self.name,
                    namespace=self.namespace,
                )
                status = pod.status
                if status.phase == "Running":
                    # Check if containers are ready
                    all_ready = all([c.ready for c in status.container_statuses])
                    if all_ready:
                        elapsed = asyncio.get_event_loop().time() - start_time
                        logging.info(
                            f"Pod '{self.name}' is ready. Time taken: {elapsed:.2f} seconds."
                        )
                        return elapsed
            except ApiException as e:
                logging.error(f"Exception when reading pod status: {e}")
                break
            await asyncio.sleep(poll_interval)

    async def destroy(self):
        await asyncio.to_thread(
            self._kubebox._core_v1.delete_namespaced_pod,
            name=self.name,
            namespace=self.namespace,
        )

    def __str__(self):
        return f"KubeboxPod(name={self.name}, namespace={self.namespace})"

    def __repr__(self):
        return str(self)


class KubeboxService:
    def __init__(self, name: str, namespace: str, kubebox: "Kubebox" = None):
        self.name = name
        self.namespace = namespace
        self._kubebox = kubebox

    async def update_network_policy(self, allowed_ips: list[str]):
        policy_name = f"{self.name}-network-policy"
        network_policy_manifest = {
            "apiVersion": "networking.k8s.io/v1",
            "kind": "NetworkPolicy",
            "metadata": {"name": policy_name},
            "spec": {
                "podSelector": {"matchLabels": {"app": self.name}},
                "policyTypes": ["Ingress", "Egress"],
                "ingress": [
                    {
                        "from": [
                            {
                                "ipBlock": {"cidr": f"{allowed_ip}/32"}
                            }
                            for allowed_ip in allowed_ips
                        ]
                    }
                ],
                "egress": [
                    {
                        "to": [
                            {
                                "ipBlock": {"cidr": f"{allowed_ip}/32"}
                            }
                            for allowed_ip in allowed_ips
                        ]
                    }
                ],
            },
        }

        try:
            api_response = await asyncio.to_thread(
                self._kubebox._networking_v1.replace_namespaced_network_policy,
                name=policy_name,
                namespace=self.namespace,
                body=network_policy_manifest
            )
            logging.info(f"Network Policy '{policy_name}' updated.")
        except ApiException as e:
            if e.status == 404:
                try:
                    api_response = await asyncio.to_thread(
                        self._kubebox._networking_v1.create_namespaced_network_policy,
                        namespace=self.namespace,
                        body=network_policy_manifest
                    )
                    logging.info(f"Network Policy '{policy_name}' created.")
                except ApiException as create_e:
                    logging.error(f"Exception when creating network policy: {create_e}")
            else:
                logging.error(f"Exception when updating network policy: {e}")

    async def wait_until_ready(self, poll_interval: float = 0.1):
        logging.info(f"Waiting for service '{self.name}' to be ready...")
        # start_time = asyncio.get_event_loop().time()
        while True:  # asyncio.get_event_loop().time() - start_time < timeout:
            try:
                # Use asyncio.to_thread to run the blocking I/O operation in a separate thread
                service = await asyncio.to_thread(
                    self._kubebox._core_v1.read_namespaced_service,
                    name=self.name,
                    namespace=self.namespace,
                )
                ingress = service.status.load_balancer.ingress
                if ingress:
                    ip = ingress[0].ip or ingress[0].hostname
                    if ip:
                        logging.info(f"Service '{self.name}' is ready with IP: {ip}.")
                        return ip
            except ApiException as e:
                logging.error(f"Exception when reading service status: {e}")
                # Consider whether to break or continue based on the type of exception

            await asyncio.sleep(poll_interval)

        raise TimeoutError(f"Timed out waiting for service '{self.name}' to be ready.")

    async def get_external_ip(self, timeout: float = 5, poll_interval: float = 0.1):
        logging.info(f"Waiting for external IP of service '{self.name}'...")
        start_time = asyncio.get_event_loop().time()

        while asyncio.get_event_loop().time() - start_time < timeout:
            try:
                # Use asyncio.to_thread to run the blocking I/O operation in a separate thread
                service = await asyncio.to_thread(
                    self._kubebox._core_v1.read_namespaced_service,
                    name=self.name,
                    namespace=self.namespace,
                )
                ingress = service.status.load_balancer.ingress
                if ingress:
                    ip = ingress[0].ip or ingress[0].hostname
                    if ip:
                        logging.info(f"Service '{self.name}' is available at {ip}.")
                        return ip
            except ApiException as e:
                logging.error(f"Exception when reading service status: {e}")
                # Consider whether to break or continue based on the type of exception

            await asyncio.sleep(poll_interval)

        raise TimeoutError(
            f"Timed out waiting for external IP of service '{self.name}'."
        )

    async def destroy(self):
        await asyncio.to_thread(
            self._kubebox._core_v1.delete_namespaced_service,
            name=self.name,
            namespace=self.namespace,
        )

    def __str__(self):
        return f"KubeboxService(name={self.name}, namespace={self.namespace})"

    def __repr__(self):
        return str(self)


class Kubebox:
    def __init__(self, kubebox_str: Optional[str] = None, terraform_path: Optional[str] = None, print_kubebox_str: bool = False):
        self.terraform_path = terraform_path
        self._load_kube_config_from_terraform(terraform_path, kubebox_str, print_kubebox_str)

        self._client = client.ApiClient()
        self._core_v1 = client.CoreV1Api()

    def create_secret(self, secret_name: str, namespace: str, data: dict[str, str]):
        # Encode the secret data in base64
        encoded_data = {k: base64.b64encode(v.encode()).decode() for k, v in data.items()}

        secret_manifest = {
            "apiVersion": "v1",
            "kind": "Secret",
            "metadata": {"name": secret_name, "namespace": namespace},
            "type": "Opaque",
            "data": encoded_data,
        }

        try:
            self._core_v1.create_namespaced_secret(namespace=namespace, body=secret_manifest)
            logging.info(f"Secret {secret_name} created successfully in namespace {namespace}")
        except ApiException as e:
            if e.status == 409:
                logging.info(f"Secret {secret_name} already exists in namespace {namespace}")
                try:
                    self._core_v1.replace_namespaced_secret(namespace=namespace, name=secret_name, body=secret_manifest)
                    logging.info(f"Secret {secret_name} updated successfully in namespace {namespace}")
                except ApiException as update_e:
                    logging.error(f"Error updating secret {secret_name}: {update_e}")
                    raise update_e
            else:
                logging.error(f"Error creating secret {secret_name}: {e}")
                raise e

    async def get_all_pods(self, namespace: str = "default") -> list[KubeboxPod]:
        pods = await asyncio.to_thread(
            self._core_v1.list_namespaced_pod, namespace=namespace
        )
        return [
            KubeboxPod(pod.metadata.name, pod.metadata.namespace, kubebox=self)
            for pod in pods.items
        ]

    async def get_all_services(
        self, namespace: str = "default"
    ) -> list[KubeboxService]:
        services = await asyncio.to_thread(
            self._core_v1.list_namespaced_service, namespace=namespace
        )
        return [
            KubeboxService(
                service.metadata.name, service.metadata.namespace, kubebox=self
            )
            for service in services.items
        ]

    def create_pod(
        self,
        pod_name: str,
        namespace: str = "default",
        image: str = "lukejagg/sandbox:latest",
        username: str = None,
        kubebox_public_key_secret_name: str = None,
        kubebox_public_key_key: str = None,
    ):
        logging.info(
            f"Creating pod: {pod_name} in namespace: {namespace} with image: {image}"
        )

        labels = {"app": pod_name}
        if username:
            labels["username"] = username

        pod_manifest = {
            "apiVersion": "v1",
            "kind": "Pod",
            "metadata": {"name": pod_name, "namespace": namespace, "labels": labels},
            "spec": {
                "containers": [
                    {
                        "name": pod_name,
                        "image": image,
                    }
                ]
            },
        }
        
        if kubebox_public_key_secret_name and kubebox_public_key_key:
            pod_manifest["spec"]["containers"][0]["env"] = [
                {
                    "name": kubebox_public_key_key,
                    "valueFrom": {
                        "secretKeyRef": {
                            "name": kubebox_public_key_secret_name,
                            "key": kubebox_public_key_key
                        }
                    }
                }
            ]

        try:
            api_response = self._core_v1.create_namespaced_pod(
                namespace=namespace, body=pod_manifest
            )
            logging.info(f"Pod {pod_name} created successfully")
            return KubeboxPod(pod_name, namespace, kubebox=self)
        except ApiException as e:
            if e.status == 409:
                logging.info(f"Pod {pod_name} already exists in namespace {namespace}")
                return KubeboxPod(pod_name, namespace, kubebox=self)
            else:
                logging.error(f"Error creating pod {pod_name}: {e}")
                raise e

    def create_service(
        self,
        pod_name: str,
        namespace: str = "default",
        username: str = None,
        ports: list[int] = [],
    ):
        logging.info(f"Creating service: {pod_name}-service in namespace: {namespace}")

        labels = {"app": pod_name}
        if username:
            labels["username"] = username

        service_name = f"{pod_name}-service"
        service_manifest = {
            "apiVersion": "v1",
            "kind": "Service",
            "metadata": {
                "name": service_name,
                "namespace": namespace,
                "labels": labels,
            },
            "spec": {
                "selector": {"app": pod_name},
                "ports": [
                    {"name": "api", "protocol": "TCP", "port": 80, "targetPort": 80},
                    *[
                        {
                            "name": f"dev-{port}",
                            "protocol": "TCP",
                            "port": port,
                            "targetPort": port,
                        }
                        for port in ports
                    ],
                ],
                "type": "LoadBalancer",
                "externalTrafficPolicy": "Local",
            },
        }

        try:
            api_response = self._core_v1.create_namespaced_service(
                namespace=namespace, body=service_manifest
            )
            logging.info(f"Service {service_name} created successfully")
            return KubeboxService(service_name, namespace, kubebox=self)
        except ApiException as e:
            if e.status == 409:
                logging.info(
                    f"Service {service_name} already exists in namespace {namespace}"
                )
                try:
                    api_response = self._core_v1.replace_namespaced_service(
                        name=service_name, namespace=namespace, body=service_manifest
                    )
                    logging.info(f"Service {service_name} updated successfully")
                    return KubeboxService(service_name, namespace, kubebox=self)
                except ApiException as update_e:
                    logging.error(f"Error updating service {service_name}: {update_e}")
                    raise update_e
            else:
                logging.error(f"Error creating service {service_name}: {e}")
                raise e
    
    def _load_kube_config_from_terraform(self, tfstate_file, kubebox_str, print_dict: bool = False):
        if tfstate_file:
            with open(tfstate_file, "r") as f:
                tfstate = json.load(f)

            # Navigate the state file to find the kubeconfig data
            resources = tfstate.get("resources", [])
            kube_config = None

            for resource in resources:
                if resource.get("type") == "azurerm_kubernetes_cluster":
                    for instance in resource.get("instances", []):
                        attributes = instance.get("attributes", {})
                        # Check for 'kube_config_raw' or 'kube_admin_config_raw'
                        kube_config_raw = attributes.get(
                            "kube_config_raw"
                        ) or attributes.get("kube_admin_config_raw")
                        if kube_config_raw:
                            # Decode the base64-encoded kubeconfig
                            # kube_config = base64.b64decode(kube_config_raw).decode('utf-8')
                            kube_config = kube_config_raw
                            break
                    if kube_config:
                        break

            if not kube_config:
                raise Exception("Failed to find kube_config in terraform state file.")

            # Write the kubeconfig to a temporary file
            kubeconfig_path = "/tmp/kubeconfig"
            with open(kubeconfig_path, "w") as f:
                f.write(kube_config)

            if print_dict:
                print(json.dumps(kube_config))

            # Load the kubeconfig
            config.load_kube_config(config_file=kubeconfig_path)
        elif kubebox_str:
            kubeconfig_path = "/tmp/kubeconfig"
            with open(kubeconfig_path, "w") as f:
                f.write(kubebox_str)
            config.load_kube_config(config_file=kubeconfig_path)
        else:
            raise Exception("No kubeconfig provided.")


if __name__ == "__main__":

    def setup_logging():
        RESET = "\033[0m"
        COLORS = {
            "DEBUG": "\033[34m",  # Blue
            "INFO": "\033[32m",  # Green
            "WARNING": "\033[33m",  # Yellow
            "ERROR": "\033[31m",  # Red
            "CRITICAL": "\033[41m",  # Red background
        }

        class CustomFormatter(logging.Formatter):
            def format(self, record):
                log_fmt = f"{COLORS.get(record.levelname, RESET)}%(asctime)s - %(levelname)s - %(message)s{RESET}"
                formatter = logging.Formatter(log_fmt, datefmt="%Y-%m-%d %H:%M:%S")
                return formatter.format(record)

        # Set up the root logger
        logging.basicConfig(level=logging.INFO)
        logger = logging.getLogger()

        # Remove default handlers and add the custom handler
        if logger.hasHandlers():
            logger.handlers.clear()

        console_handler = logging.StreamHandler()
        console_handler.setFormatter(CustomFormatter())
        logger.addHandler(console_handler)

    async def main(name: str, username: str):    
        from dotenv import load_dotenv
        import os

        load_dotenv()
        secret = os.getenv("KUBEBOX_CONFIG")
        
        KUBEBOX_PUBLIC_KEY = os.getenv("KUBEBOX_PUBLIC_KEY")
        KUBEBOX_PRIVATE_KEY = os.getenv("KUBEBOX_PRIVATE_KEY")
        
        
        from kubebox.security import sign_packet, verify_packet, encrypt_packet, decrypt_packet
        
        packet = b"Important data from API"
        signature = sign_packet(packet, KUBEBOX_PRIVATE_KEY)
        is_verified = verify_packet(packet, signature, KUBEBOX_PUBLIC_KEY)
        print("Packet is verified as coming from the API:", is_verified)
        
        # SANDBOX_PUBLIC_KEY = os.getenv("SANDBOX_PUBLIC_KEY")
        # SANDBOX_PRIVATE_KEY = os.getenv("REMOVE_THIS_SANDBOX_PRIVATE_KEY")
        # encrypted_packet = encrypt_packet(packet, SANDBOX_PUBLIC_KEY)
        # decrypted_packet = decrypt_packet(encrypted_packet, SANDBOX_PRIVATE_KEY)
        # print("Decrypted packet:", decrypted_packet)
        # return

        # kubebox = Kubebox(terraform_path="../../apps/sandbox/terraform.tfstate", print_kubebox_str=True)
        kubebox = Kubebox(secret)
        kubebox.create_secret(secret_name="kubebox-public-key", namespace="default", data={"KUBEBOX_PUBLIC_KEY": KUBEBOX_PUBLIC_KEY})
        
        pod = kubebox.create_pod(name, username=username, kubebox_public_key_secret_name="kubebox-public-key", kubebox_public_key_key="KUBEBOX_PUBLIC_KEY")
        service = kubebox.create_service(name, username=username)

        await pod.wait_until_ready()
        await service.wait_until_ready()
        ip = await service.get_external_ip()
        print(f"http://{ip}")

        # pods = await kubebox.get_all_pods()
        # services = await kubebox.get_all_services()
        # print(pods)
        # print(services)

        # for pod in pods:
        #     await pod.destroy()

        # for service in services:
        #     await service.destroy()

    setup_logging()
    asyncio.run(main("luke-pod", "luke"))



from ._client import (
    SandboxClient,
    CommandMode,
    CommandOutput,
    CommandExit,
    CommandResult,
    Status,
    CommandKilled,
    CommandError,
    BackgroundProcess,
)

from ._manager import (
    Kubebox,
    KubeboxPod,
    KubeboxService,
    KubeboxPodExistsError,
)

__all__ = [
    "SandboxClient",
    "CommandMode",
    "CommandOutput",
    "CommandExit",
    "CommandResult",
    "Status",
    "CommandKilled",
    "CommandError",
    "BackgroundProcess",
    "Kubebox",
    "KubeboxPod",
    "KubeboxService",
    "KubeboxPodExistsError",
]
